Imports System
Imports System.IO
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Globalization
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports Microsoft.ReportingServices.Interfaces
Imports System.Xml

Namespace MSSQLRS.FormsSecurity
    Public Class AuthorizationExtension : Implements IAuthorizationExtension
        ' The sample is based on Microsoft's Forms Authentication Sample for Reporting Services. 
        ' The original sample was written in C#. It has been translated it into Visual Basic 
        ' for consistency with the other examples in this book. Some revisions were also made 
        ' to better fit with the Galactic Delivery Services examples.

        Private Shared m_adminUserName As String

        Shared Sub New()
            InitializeMaps()
        End Sub

        ' This function returns a security descriptor that is store with an
        ' individual item in the Report Server database.
        Public Function CreateSecurityDescriptor(ByVal acl As AceCollection, ByVal itemType As SecurityItemType, ByRef stringSecDesc As String) As Byte() Implements IAuthorizationExtension.CreateSecurityDescriptor
            Dim bf As BinaryFormatter = New BinaryFormatter
            Dim result As MemoryStream = New MemoryStream

            ' Creates a memory stream and serializes the ACL for storage.
            bf.Serialize(result, acl)
            stringSecDesc = Nothing

            Return result.GetBuffer

        End Function

        ' This function determines whether a given user is authorized to 
        ' access the item for a given catalog operation.
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As CatalogOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As CatalogOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.CatalogOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' Overload for an array of catalog operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperations As CatalogOperation()) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim operation As CatalogOperation

            For Each operation In requiredOperations
                If Not CheckAccess(userName, userToken, secDesc, operation) Then
                    Return False
                End If
            Next

            Return True

        End Function

        ' Overload for report operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As ReportOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As ReportOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.ReportOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' Overload for folder operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As FolderOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As FolderOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.FolderOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' Overload for an array of folder operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperations As FolderOperation()) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim operation As FolderOperation

            For Each operation In requiredOperations
                If Not CheckAccess(userName, userToken, secDesc, operation) Then
                    Return False
                End If
            Next

            Return True

        End Function

        ' Overload for resource operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As ResourceOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As ResourceOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.ResourceOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' Overload for an array of resource operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperations As ResourceOperation()) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim operation As ResourceOperation

            For Each operation In requiredOperations
                If Not CheckAccess(userName, userToken, secDesc, operation) Then
                    Return False
                End If
            Next

            Return True

        End Function

        ' Overload for data source operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As DatasourceOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As DatasourceOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.DatasourceOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' Overload for model item operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As ModelItemOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As ModelItemOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.ModelItemOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' Overload for model operations
        Public Function CheckAccess(ByVal userName As String, ByVal userToken As IntPtr, ByVal secDesc() As Byte, ByVal requiredOperation As ModelOperation) As Boolean Implements IAuthorizationExtension.CheckAccess
            Dim acl As AceCollection
            Dim ace As AceStruct
            Dim aclOperation As ModelOperation

            ' If the user is the administrator, allow unrestricted access.
            ' Because SQL Server defaults to case-insensitive, we have to
            ' perform a case insensitive comparison.
            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                Return True
            End If

            acl = DeserializeACL(secDesc)
            For Each ace In acl
                ' First check to see if the user has an access control
                ' entry for the item.
                If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                    ' If an entry is found, return true if the given
                    ' required operation is contained in the ACE structure
                    For Each aclOperation In ace.ModelOperations
                        If aclOperation = requiredOperation Then
                            Return True
                        End If
                    Next
                End If
            Next

            Return False

        End Function

        ' This function returns a set of permissions a specific user has for
        ' a specific item managed in the Report Server database. This function 
        ' provides the underlying support for the web service method GetPermissions().
        Public Function GetPermissions(ByVal userName As String, ByVal userToken As IntPtr, ByVal itemType As SecurityItemType, ByVal secDesc() As Byte) As StringCollection Implements IAuthorizationExtension.GetPermissions
            Dim permissions As StringCollection = New StringCollection
            Dim catOper As CatalogOperation
            Dim rptOper As ReportOperation
            Dim fldOper As FolderOperation
            Dim resOper As ResourceOperation
            Dim dsOper As DatasourceOperation
            Dim acl As AceCollection
            Dim ace As AceStruct

            If String.Compare(userName, m_adminUserName, True, CultureInfo.CurrentCulture) = 0 Then
                For Each catOper In m_CatOperNames.Keys
                    If Not permissions.Contains(CStr(m_CatOperNames(catOper))) Then
                        permissions.Add(CStr(m_CatOperNames(catOper)))
                    End If
                Next

                For Each rptOper In m_RptOperNames.Keys
                    If Not permissions.Contains(CStr(m_RptOperNames(rptOper))) Then
                        permissions.Add(CStr(m_RptOperNames(rptOper)))
                    End If
                Next

                For Each fldOper In m_FldOperNames.Keys
                    If Not permissions.Contains(CStr(m_FldOperNames(fldOper))) Then
                        permissions.Add(CStr(m_FldOperNames(fldOper)))
                    End If
                Next

                For Each resOper In m_ResOperNames.Keys
                    If Not permissions.Contains(CStr(m_ResOperNames(resOper))) Then
                        permissions.Add(CStr(m_ResOperNames(resOper)))
                    End If
                Next

                For Each dsOper In m_DSOperNames.Keys
                    If Not permissions.Contains(CStr(m_DSOperNames(dsOper))) Then
                        permissions.Add(CStr(m_DSOperNames(dsOper)))
                    End If
                Next
            Else
                acl = DeserializeACL(secDesc)
                For Each ace In acl
                    If String.Compare(userName, ace.PrincipalName, True, CultureInfo.CurrentCulture) = 0 Then
                        For Each catOper In ace.CatalogOperations
                            If Not permissions.Contains(CStr(m_CatOperNames(catOper))) Then
                                permissions.Add(CStr(m_CatOperNames(catOper)))
                            End If
                        Next

                        For Each rptOper In ace.ReportOperations
                            If Not permissions.Contains(CStr(m_RptOperNames(rptOper))) Then
                                permissions.Add(CStr(m_RptOperNames(rptOper)))
                            End If
                        Next

                        For Each fldOper In ace.FolderOperations
                            If Not permissions.Contains(CStr(m_FldOperNames(fldOper))) Then
                                permissions.Add(CStr(m_FldOperNames(fldOper)))
                            End If
                        Next

                        For Each resOper In ace.ResourceOperations
                            If Not permissions.Contains(CStr(m_ResOperNames(resOper))) Then
                                permissions.Add(CStr(m_ResOperNames(resOper)))
                            End If
                        Next

                        For Each dsOper In ace.DatasourceOperations
                            If Not permissions.Contains(CStr(m_DSOperNames(dsOper))) Then
                                permissions.Add(CStr(m_DSOperNames(dsOper)))
                            End If
                        Next
                    End If
                Next
            End If

            Return permissions

        End Function

        ' This function is used to deserializ the ACL that is
        ' stored by the Report Server.
        Private Function DeserializeACL(ByVal secDesc() As Byte) As AceCollection
            Dim bf As BinaryFormatter = New BinaryFormatter
            Dim sdStream As MemoryStream
            Dim acl As AceCollection

            If Not secDesc Is Nothing Then
                sdStream = New MemoryStream(secDesc)
                acl = bf.Deserialize(sdStream)
            Else
                acl = New AceCollection
            End If

            Return acl
        End Function

        Private Shared m_CatOperNames As Hashtable
        Private Shared m_FldOperNames As Hashtable
        Private Shared m_RptOperNames As Hashtable
        Private Shared m_ResOperNames As Hashtable
        Private Shared m_DSOperNames As Hashtable

        Private Const NrRptOperations As Integer = 27
        Private Const NrFldOperations As Integer = 9
        Private Const NrResOperations As Integer = 7
        Private Const NrDSOperations As Integer = 7
        Private Const NrCatOperations As Integer = 15

        ' This subroutine is used to create mappings to the various
        ' operations in Reporting Services. These mappings support
        ' the implementation of the GetPermissions method.
        Private Shared Sub InitializeMaps()
            ' Create the Operation Names data

            'Catalog Operations
            m_CatOperNames = New Hashtable
            m_CatOperNames.Add(CatalogOperation.CreateRoles, OperationNames.OperCreateRoles)
            m_CatOperNames.Add(CatalogOperation.DeleteRoles, OperationNames.OperDeleteRoles)
            m_CatOperNames.Add(CatalogOperation.ReadRoleProperties, OperationNames.OperReadRoleProperties)
            m_CatOperNames.Add(CatalogOperation.UpdateRoleProperties, OperationNames.OperUpdateRoleProperties)
            m_CatOperNames.Add(CatalogOperation.ReadSystemProperties, OperationNames.OperReadSystemProperties)
            m_CatOperNames.Add(CatalogOperation.UpdateSystemProperties, OperationNames.OperUpdateSystemProperties)
            m_CatOperNames.Add(CatalogOperation.GenerateEvents, OperationNames.OperGenerateEvents)
            m_CatOperNames.Add(CatalogOperation.ReadSystemSecurityPolicy, OperationNames.OperReadSystemSecurityPolicy)
            m_CatOperNames.Add(CatalogOperation.UpdateSystemSecurityPolicy, OperationNames.OperUpdateSystemSecurityPolicy)
            m_CatOperNames.Add(CatalogOperation.CreateSchedules, OperationNames.OperCreateSchedules)
            m_CatOperNames.Add(CatalogOperation.DeleteSchedules, OperationNames.OperDeleteSchedules)
            m_CatOperNames.Add(CatalogOperation.ReadSchedules, OperationNames.OperReadSchedules)
            m_CatOperNames.Add(CatalogOperation.UpdateSchedules, OperationNames.OperUpdateSchedules)
            m_CatOperNames.Add(CatalogOperation.ListJobs, OperationNames.OperListJobs)
            m_CatOperNames.Add(CatalogOperation.CancelJobs, OperationNames.OperCancelJobs)

            If m_CatOperNames.Count <> NrCatOperations Then
                Throw New Exception("Number of catalog names does not match.")
            End If

            ' Folder Operations
            m_FldOperNames = New Hashtable
            m_FldOperNames.Add(FolderOperation.CreateFolder, OperationNames.OperCreateFolder)
            m_FldOperNames.Add(FolderOperation.Delete, OperationNames.OperDelete)
            m_FldOperNames.Add(FolderOperation.ReadProperties, OperationNames.OperReadProperties)
            m_FldOperNames.Add(FolderOperation.UpdateProperties, OperationNames.OperUpdateProperties)
            m_FldOperNames.Add(FolderOperation.CreateReport, OperationNames.OperCreateReport)
            m_FldOperNames.Add(FolderOperation.CreateResource, OperationNames.OperCreateResource)
            m_FldOperNames.Add(FolderOperation.ReadAuthorizationPolicy, OperationNames.OperReadAuthorizationPolicy)
            m_FldOperNames.Add(FolderOperation.UpdateDeleteAuthorizationPolicy, OperationNames.OperUpdateDeleteAuthorizationPolicy)
            m_FldOperNames.Add(FolderOperation.CreateDatasource, OperationNames.OperCreateResource)

            If m_FldOperNames.Count <> NrFldOperations Then
                Throw New Exception("Number of folder names does not match.")
            End If

            ' Report Operations
            m_RptOperNames = New Hashtable
            m_RptOperNames.Add(ReportOperation.Delete, OperationNames.OperDelete)
            m_RptOperNames.Add(ReportOperation.ReadProperties, OperationNames.OperReadProperties)
            m_RptOperNames.Add(ReportOperation.UpdateProperties, OperationNames.OperUpdateProperties)
            m_RptOperNames.Add(ReportOperation.UpdateParameters, OperationNames.OperUpdateParameters)
            m_RptOperNames.Add(ReportOperation.ReadDatasource, OperationNames.OperReadDatasources)
            m_RptOperNames.Add(ReportOperation.UpdateDatasource, OperationNames.OperUpdateDatasources)
            m_RptOperNames.Add(ReportOperation.ReadReportDefinition, OperationNames.OperReadReportDefinition)
            m_RptOperNames.Add(ReportOperation.UpdateReportDefinition, OperationNames.OperUpdateReportDefinition)
            m_RptOperNames.Add(ReportOperation.CreateSubscription, OperationNames.OperCreateSubscription)
            m_RptOperNames.Add(ReportOperation.DeleteSubscription, OperationNames.OperDeleteSubscription)
            m_RptOperNames.Add(ReportOperation.ReadSubscription, OperationNames.OperReadSubscription)
            m_RptOperNames.Add(ReportOperation.UpdateSubscription, OperationNames.OperUpdateSubscription)
            m_RptOperNames.Add(ReportOperation.CreateAnySubscription, OperationNames.OperCreateAnySubscription)
            m_RptOperNames.Add(ReportOperation.DeleteAnySubscription, OperationNames.OperDeleteAnySubscription)
            m_RptOperNames.Add(ReportOperation.ReadAnySubscription, OperationNames.OperReadAnySubscription)
            m_RptOperNames.Add(ReportOperation.UpdateAnySubscription, OperationNames.OperUpdateAnySubscription)
            m_RptOperNames.Add(ReportOperation.UpdatePolicy, OperationNames.OperUpdatePolicy)
            m_RptOperNames.Add(ReportOperation.ReadPolicy, OperationNames.OperReadPolicy)
            m_RptOperNames.Add(ReportOperation.DeleteHistory, OperationNames.OperDeleteHistory)
            m_RptOperNames.Add(ReportOperation.ListHistory, OperationNames.OperListHistory)
            m_RptOperNames.Add(ReportOperation.ExecuteAndView, OperationNames.OperExecuteAndView)
            m_RptOperNames.Add(ReportOperation.CreateResource, OperationNames.OperCreateResource)
            m_RptOperNames.Add(ReportOperation.CreateSnapshot, OperationNames.OperCreateSnapshot)
            m_RptOperNames.Add(ReportOperation.ReadAuthorizationPolicy, OperationNames.OperReadAuthorizationPolicy)
            m_RptOperNames.Add(ReportOperation.UpdateDeleteAuthorizationPolicy, OperationNames.OperUpdateDeleteAuthorizationPolicy)
            m_RptOperNames.Add(ReportOperation.Execute, OperationNames.OperExecute)
            m_RptOperNames.Add(ReportOperation.CreateLink, OperationNames.OperCreateLink)

            If m_RptOperNames.Count <> NrRptOperations Then
                Throw New Exception("Number of report names does not match.")
            End If

            ' Resource Operations
            m_ResOperNames = New Hashtable
            m_ResOperNames.Add(ResourceOperation.Delete, OperationNames.OperDelete)
            m_ResOperNames.Add(ResourceOperation.ReadProperties, OperationNames.OperReadProperties)
            m_ResOperNames.Add(ResourceOperation.UpdateProperties, OperationNames.OperUpdateProperties)
            m_ResOperNames.Add(ResourceOperation.ReadContent, OperationNames.OperReadContent)
            m_ResOperNames.Add(ResourceOperation.UpdateContent, OperationNames.OperUpdateContent)
            m_ResOperNames.Add(ResourceOperation.ReadAuthorizationPolicy, OperationNames.OperReadAuthorizationPolicy)
            m_ResOperNames.Add(ResourceOperation.UpdateDeleteAuthorizationPolicy, OperationNames.OperUpdateDeleteAuthorizationPolicy)

            If m_ResOperNames.Count <> NrResOperations Then
                Throw New Exception("Number of resource names does not match.")
            End If

            ' Data Source Operations
            m_DSOperNames = New Hashtable
            m_DSOperNames.Add(DatasourceOperation.Delete, OperationNames.OperDelete)
            m_DSOperNames.Add(DatasourceOperation.ReadProperties, OperationNames.OperReadProperties)
            m_DSOperNames.Add(DatasourceOperation.UpdateProperties, OperationNames.OperUpdateProperties)
            m_DSOperNames.Add(DatasourceOperation.ReadContent, OperationNames.OperReadContent)
            m_DSOperNames.Add(DatasourceOperation.UpdateContent, OperationNames.OperUpdateContent)
            m_DSOperNames.Add(DatasourceOperation.ReadAuthorizationPolicy, OperationNames.OperReadAuthorizationPolicy)
            m_DSOperNames.Add(DatasourceOperation.UpdateDeleteAuthorizationPolicy, OperationNames.OperUpdateDeleteAuthorizationPolicy)

            If m_DSOperNames.Count <> NrDSOperations Then
                Throw New Exception("Number of data source names does not match.")
            End If

        End Sub

        ' This subroutine implements SetConfiguration as required by IExtension
        Public Sub SetConfiguration(ByVal configuration As String) Implements IAuthorizationExtension.SetConfiguration
            ' Retrieve the admin user and password from the config settings
            ' and verify it.
            Dim doc As XmlDocument = New XmlDocument
            Dim child As XmlNode

            doc.LoadXml(configuration)

            If doc.DocumentElement.Name = "AdminConfiguration" Then
                For Each child In doc.DocumentElement.ChildNodes
                    If child.Name = "UserName" Then
                        m_adminUserName = child.InnerText
                    Else
                        Throw New Exception("Unrecognized configuration element.")
                    End If
                Next

                If MSSQLRS.FormsSecurity.AuthenticationExtension.VerifyUser(m_adminUserName) = False Then
                    Throw New Exception("An attempt was made to load an Administrative user for the Report Server that is not valid.")
                End If
            Else
                Throw New Exception("Error loading config data.")
            End If
        End Sub

        ' LocalizedName is required by the implementation of IExtension
        Public ReadOnly Property LocalizedName() As String Implements IExtension.LocalizedName
            Get
                Return Nothing
            End Get
        End Property

    End Class

End Namespace
